#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
A portable version of ``six``, provide basic python2/3 compatible utilities.
"""

import sys

if sys.version_info[0] == 3:
    str_type = str
    int_type = (int,)
    pk_protocol = 3
    is_py2 = False
    is_py3 = True
else:
    str_type = basestring
    int_type = (int, long)
    pk_protocol = 2
    is_py2 = True
    is_py3 = False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
2016年还在由活跃的社区维护的rsa扩展有 ``cryptography`` 和 ``rsa``。 cryp集合了非常
多主流的加密算法, 并且都做了非常多的安全和性能优化(C语言实现)。而rsa是纯Python
实现, 安装非常方便, 使用起来也较为简单。所以本模块最后决定使用rsa进行开发。
"""

from __future__ import print_function, unicode_literals

import os
from os.path import join

import rsa
from rsa.bigfile import *

from windtalker.cipher import BaseCipher
from windtalker.exc import PasswordError
from windtalker import fingerprint
from windtalker import files
from windtalker import py23


class AsymmetricCipher(BaseCipher):
    """A asymmtric encryption algorithm utility class helps you easily
    encrypt/decrypt text and files.

    :param my_pubkey: your public key
    :param my_privkey: your private key
    :param his_pubkey: other's public key you use to encrypt message

    **中文文档**

    非对称加密器。
    """
    # key length/max length msg, 512/53, 1024/117, 2045/245
    _encrypt_chunk_size = 53
    _decrypt_chunk_size = 53

    def __init__(self, my_pubkey, my_privkey, his_pubkey):
        self.my_pubkey = my_pubkey
        self.my_privkey = my_privkey
        self.his_pubkey = his_pubkey

    @staticmethod
    def newkeys(nbits=1024):
        """Create a new pair of public and private key pair to use.
        """
        pubkey, privkey = rsa.newkeys(nbits, poolsize=1)
        return pubkey, privkey

    def encrypt(self, binary, use_sign=True):
        """Encrypt binary data.

        **中文文档**

        - 发送消息时只需要对方的pubkey
        - 如需使用签名, 则双方都需要持有对方的pubkey
        """
        token = rsa.encrypt(binary, self.his_pubkey)  # encrypt it
        if use_sign:
            self.sign = rsa.sign(binary, self.my_privkey, "SHA-1")  # sign it
        return token

    def decrypt(self, token, signature=None):
        """Decrypt binary data.

        **中文文档**

        - 接收消息时只需要自己的privkey
        - 如需使用签名, 则双方都需要持有对方的pubkey
        """
        binary = rsa.decrypt(token, self.my_privkey)
        if signature:
            rsa.verify(binary, signature, self.his_pubkey)
        return binary

    def encrypt_file(self, path, output_path=None,
                     overwrite=False,
                     enable_verbose=True):
        """Encrypt a file using rsa.

        RSA for big file encryption is very slow. For big file, use
        symmetric encryption and use RSA to encrypt the password please.
        """
        path = os.path.abspath(path)

        if not output_path:
            output_path = files.get_encrypted_file_path(path)

        if not overwrite:
            if os.path.exists(output_path):
                raise FileExistsError(
                    "output path '%s' already exists.." % output_path)

        with open(path, "rb") as infile, open(output_path, "wb") as outfile:
            encrypt_bigfile(infile, outfile, self.his_pubkey)

    def decrypt_file(self, path, output_path=None,
                     overwrite=False,
                     enable_verbose=True):
        """Decrypt a file using rsa.
        """
        path = os.path.abspath(path)

        if not output_path:
            output_path = files.recover_path(path)

        if not overwrite:
            if os.path.exists(output_path):
                raise FileExistsError(
                    "output path '%s' already exists.." % output_path)

        with open(path, "rb") as infile, open(output_path, "wb") as outfile:
            decrypt_bigfile(infile, outfile, self.my_privkey)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
This module is built on Python standard hashlib, provides utility method
to find hash value for a bytes, a string, a Python object or a file.

Import this module::

    >>> from fingerprint import fingerprint

Example::

    >>> fingerprint.of_bytes(bytes(16))
    >>> fingerprint.of_text("Hello World")
    >>> fingerprint.of_pyobj(dict(a=1, b=2, c=3))
    >>> fingerprint.of_file("fingerprint.py")

You can switch the hash algorithm to use::

    >>> fingerprint.use("md5") # also "sha1", "sha256", "sha512"


**中文文档**

本模块提供了一些计算Hash值的简便方法。对于 of_pyobj()方法来说, 请注意在读写时
均使用相同的Python大版本(2/3)。
"""

import os
import sys
import pickle
import hashlib

is_py2 = (sys.version_info[0] == 2)
if is_py2:
    pk_protocol = 2
else:
    pk_protocol = 3


class FingerPrint(object):
    """A hashlib wrapper class allow you to use one line to do hash as you wish.

    Usage::

        >>> from fingerprint import fingerprint
        >>> print(fingerprint.of_bytes(bytes(123)))
        b1fec41621e338896e2d26f232a6b006

        >>> print(fingerprint.of_text("message"))
        78e731027d8fd50ed642340b7c9a63b3

        >>> print(fingerprint.of_pyobj({"key": "value"}))
        4c502ab399c89c8758a2d8c37be98f69

        >>> print(fingerprint.of_file("fingerprint.py"))
        4cddcb5562cbff652b0e4c8a0300337a
    """
    _mapper = {
        "md5": hashlib.md5,
        "sha1": hashlib.sha1,
        "sha256": hashlib.sha256,
        "sha512": hashlib.sha512,
    }

    def __init__(self, algorithm="md5", pk_protocol=pk_protocol):
        self.use(algorithm)
        self.set_return_str()
        self.set_pickle_protocol(pk_protocol)

    def digest_to_int(self, digest):
        """Convert hexdigest str to int.
        """
        return int(digest, 16)

    def use(self, algorithm):
        """Change the hash algorithm you gonna use.
        """
        try:
            self.hash_algo = self._mapper[algorithm.strip().lower()]
        except IndexError:
            template = "'%s' is not supported, try one of %s."
            raise ValueError(template % (algorithm, list(self._mapper)))

    def set_return_int(self):
        """Set to return hex integer.
        """
        self.return_int = True

    def set_return_str(self):
        """Set to return hex string.
        """
        self.return_int = False

    def set_pickle_protocol(self, pk_protocol):
        """Set pickle protocol.
        """
        if pk_protocol not in [2, 3]:
            raise ValueError("pickle protocol has to be 2 or 3!")
        self.pk_protocol = pk_protocol

    def of_bytes(self, py_bytes):
        """Use default hash method to return hash value of bytes.
        """
        m = self.hash_algo()
        m.update(py_bytes)
        if self.return_int:
            return int(m.hexdigest(), 16)
        else:
            return m.hexdigest()

    def of_text(self, text, encoding="utf-8"):
        """Use default hash method to return hash value of a piece of string
        default setting use 'utf-8' encoding.
        """
        m = self.hash_algo()
        m.update(text.encode(encoding))
        if self.return_int:
            return int(m.hexdigest(), 16)
        else:
            return m.hexdigest()

    def of_pyobj(self, pyobj):
        """Use default hash method to return hash value of a piece of Python
        picklable object.
        """
        m = self.hash_algo()
        m.update(pickle.dumps(pyobj, protocol=self.pk_protocol))
        if self.return_int:
            return int(m.hexdigest(), 16)
        else:
            return m.hexdigest()

    def of_file(self, abspath, nbytes=0, chunk_size=1024):
        """Use default hash method to return hash value of a piece of a file

        Estimate processing time on:

        :param abspath: the absolute path to the file.
        :param nbytes: only has first N bytes of the file. if 0, hash all file.
        :param chunk_size: The max memory we use at one time.

        CPU = i7-4600U 2.10GHz - 2.70GHz, RAM = 8.00 GB
        1 second can process 0.25GB data

        - 0.59G - 2.43 sec
        - 1.3G - 5.68 sec
        - 1.9G - 7.72 sec
        - 2.5G - 10.32 sec
        - 3.9G - 16.0 sec

        ATTENTION:
            if you change the meta data (for example, the title, years
            information in audio, video) of a multi-media file, then the hash
            value gonna also change.
        """
        if nbytes < 0:
            raise ValueError("chunk_size cannot smaller than 0")
        if chunk_size < 1:
            raise ValueError("chunk_size cannot smaller than 1")
        if (nbytes > 0) and (nbytes < chunk_size):
            chunk_size = nbytes

        m = self.hash_algo()
        with open(abspath, "rb") as f:
            if nbytes:  # use first n bytes
                have_reads = 0
                while True:
                    have_reads += chunk_size
                    if have_reads > nbytes:
                        n = nbytes - (have_reads - chunk_size)
                        if n:
                            data = f.read(n)
                            m.update(data)
                        break
                    else:
                        data = f.read(chunk_size)
                        m.update(data)
            else:  # use entire content
                while True:
                    data = f.read(chunk_size)
                    if not data:
                        break
                    m.update(data)

        return m.hexdigest()


fingerprint = FingerPrint()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function, unicode_literals

import io
import os
import time
import base64

from windtalker import files


class BaseCipher(object):
    """
    Base cipher class.

    Any cipher utility class that using any encryption algorithm can be
    inherited from this base class. The only method you need to implement is
    :meth:`BaseCipher.encrypt` and :meth:`BaseCipher.decrypt`. Because once you
    can encrypt binary data, then you can encrypt text, file, and directory.
    """
    _encrypt_chunk_size = 1024
    _decrypt_chunk_size = 1024

    def b64encode_str(self, text):
        """base64 encode a text, return string also.
        """
        return base64.b64encode(text.encode("utf-8")).decode("utf-8")

    def b64decode_str(self, text):
        """base64 decode a text, return string also.
        """
        return base64.b64decode(text.encode("utf-8")).decode("utf-8")

    def encrypt(self, binary, *args, **kwargs):
        """Overwrite this method using your encrypt algorithm.

        :param binary: binary data you need to encrypt
        :return: encrypted_binary, encrypted binary data
        """
        raise NotImplementedError

    def decrypt(self, binary, *args, **kwargs):
        """Overwrite this method using your decrypt algorithm.

        :param binary: binary data you need to decrypt
        :return: decrypted_binary, decrypted binary data
        """
        raise NotImplementedError

    def encrypt_binary(self, binary, *args, **kwargs):
        """
        input: bytes, output: bytes
        """
        return self.encrypt(binary, *args, **kwargs)

    def decrypt_binary(self, binary, *args, **kwargs):
        """
        input: bytes, output: bytes
        """
        return self.decrypt(binary, *args, **kwargs)

    def encrypt_text(self, text, *args, **kwargs):
        """
        Encrypt a string.

        input: unicode str, output: unicode str
        """
        b = text.encode("utf-8")
        token = self.encrypt(b, *args, **kwargs)
        return base64.b64encode(token).decode("utf-8")

    def decrypt_text(self, text, *args, **kwargs):
        """
        Decrypt a string.

        input: unicode str, output: unicode str
        """
        b = text.encode("utf-8")
        token = base64.b64decode(b)
        return self.decrypt(token, *args, **kwargs).decode("utf-8")

    def _show(self, message, indent=0, enable_verbose=True):
        """Message printer.
        """
        if enable_verbose:
            print("    " * indent + message)

    def encrypt_file(self, path, output_path=None,
                     overwrite=False,
                     stream=True,
                     enable_verbose=True):
        """Encrypt a file. If output_path are not given, then try to use the
        path with a surfix appended. The default automatical file path handling
        is defined here :meth:`windtalker.files.get_encrypted_file_path`

        :param path: path of the file you need to encrypt
        :param output_path: encrypted file output path
        :param overwrite: if True, then silently overwrite output file if exists
        :param stream: if it is a very big file, stream mode can avoid using
          too much memory
        :param enable_verbose: boolean, trigger on/off the help information
        """
        path = os.path.abspath(path)

        if not output_path:
            output_path = files.get_encrpyted_path(path)

        if not overwrite:  # pragma: no cover
            if os.path.exists(output_path):
                raise EnvironmentError(
                    "output path '%s' already exists.." % output_path)

        self._show("Encrypt '%s' ..." % path, enable_verbose=enable_verbose)
        st = time.clock()
        files.transform(path, output_path, converter=self.encrypt,
                        overwrite=overwrite, stream=stream,
                        chunksize=self._encrypt_chunk_size)
        self._show("    Finished! Elapse %.6f seconds" % (time.clock() - st,),
                   enable_verbose=enable_verbose)

        return output_path

    def decrypt_file(self, path, output_path=None,
                     overwrite=False,
                     stream=True,
                     enable_verbose=True):
        """Decrypt a file. If output_path are not given, then try to use the
        path with a surfix appended. The default automatical file path handling
        is defined here :meth:`windtalker.files.recover_path`

        :param path: path of the file you need to decrypt
        :param output_path: decrypted file output path
        :param overwrite: if True, then silently overwrite output file if exists
        :param stream: if it is a very big file, stream mode can avoid using
          too much memory
        :param enable_verbose: boolean, trigger on/off the help information
        """
        path = os.path.abspath(path)

        if not output_path:
            output_path = files.recover_path(path)

        if not overwrite:  # pragma: no cover
            if os.path.exists(output_path):
                raise EnvironmentError(
                    "output path '%s' already exists.." % output_path)

        self._show("Decrypt '%s' ..." % path, enable_verbose=enable_verbose)
        st = time.clock()
        files.transform(path, output_path, converter=self.decrypt,
                        overwrite=overwrite, stream=stream,
                        chunksize=self._decrypt_chunk_size)
        self._show("    Finished! Elapse %.6f seconds" % (time.clock() - st,),
                   enable_verbose=enable_verbose)

        return output_path

    def encrypt_dir(self, path, output_path=None,
                    overwrite=False,
                    stream=True,
                    enable_verbose=True):
        """Encrypt everything in a directory.

        :param path: path of the dir you need to encrypt
        :param output_path: encrypted dir output path
        :param overwrite: if True, then silently overwrite output file if exists
        :param stream: if it is a very big file, stream mode can avoid using
          too much memory
        :param enable_verbose: boolean, trigger on/off the help information
        """
        path = os.path.abspath(path)

        if not output_path:
            output_path = files.get_encrpyted_path(path)

        if not overwrite:  # pragma: no cover
            if os.path.exists(output_path):
                raise EnvironmentError(
                    "output path '%s' already exists.." % output_path)

        self._show("--- Encrypt directory '%s' ---" % path,
                   enable_verbose=enable_verbose)
        st = time.clock()
        for current_dir, _, file_list in os.walk(path):
            new_dir = current_dir.replace(path, output_path)
            if not os.path.exists(new_dir):
                os.mkdir(new_dir)
            for basename in file_list:
                old_path = os.path.join(current_dir, basename)
                new_path = os.path.join(new_dir, basename)
                self.encrypt_file(old_path, new_path,
                                  overwrite=overwrite,
                                  stream=stream,
                                  enable_verbose=enable_verbose)
        self._show("Complete! Elapse %.6f seconds" % (time.clock() - st,),
                   enable_verbose=enable_verbose)
        return output_path

    def decrypt_dir(self, path, output_path=None,
                    overwrite=False,
                    stream=True,
                    enable_verbose=True):
        """Decrypt everything in a directory.

        :param path: path of the dir you need to decrypt
        :param output_path: decrypted dir output path
        :param overwrite: if True, then silently overwrite output file if exists
        :param stream: if it is a very big file, stream mode can avoid using
          too much memory
        :param enable_verbose: boolean, trigger on/off the help information
        """
        path = os.path.abspath(path)

        if not output_path:
            output_path = files.recover_path(path)

        if not overwrite:  # pragma: no cover
            if os.path.exists(output_path):
                raise EnvironmentError(
                    "output path '%s' already exists.." % output_path)

        self._show("--- Decrypt directory '%s' ---" % path,
                   enable_verbose=enable_verbose)
        st = time.clock()
        for current_dir, _, file_list in os.walk(path):
            new_dir = current_dir.replace(path, output_path)
            if not os.path.exists(new_dir):
                os.mkdir(new_dir)
            for basename in file_list:
                old_path = os.path.join(current_dir, basename)
                new_path = os.path.join(new_dir, basename)
                self.decrypt_file(old_path, new_path,
                                  overwrite=overwrite,
                                  stream=stream,
                                  enable_verbose=enable_verbose)
        self._show("Complete! Elapse %.6f seconds" % (time.clock() - st,),
                   enable_verbose=enable_verbose)

        return output_path



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pathlib_mate import Path


default_surfix = "-encrypted"  # windtalker secret


def get_encrpyted_path(original_path, surfix=default_surfix):
    r"""
    Find the output encrypted file /dir path (by adding a surfix).

    Example:

    - ``C:\test.txt`` -> ``C:\test-encrypted.txt``
    - ``C:\Users\admin\Documents`` -> ``C:\Users\admin\Documents-encrypted``
    """
    p = Path(original_path).absolute()
    encrypted_p = p.change(new_fname=p.fname + surfix)
    return encrypted_p.abspath


def recover_path(encrypted_path, surfix=default_surfix):
    r"""
    Find the original path of encrypted file or dir.

    Example:

    - ``C:\test-encrypted.txt`` -> ``C:\test.txt``
    - ``C:\Users\admin\Documents-encrypted`` -> ``C:\Users\admin\Documents``
    """
    surfix_reversed = surfix[::-1]

    p = Path(encrypted_path).absolute()
    fname = p.fname
    fname_reversed = fname[::-1]
    new_fname = fname_reversed.replace(surfix_reversed, "", 1)[::-1]
    decrypted_p = p.change(new_fname=new_fname)
    return decrypted_p.abspath


def transform(src, dst, converter,
              overwrite=False, stream=True, chunksize=1024**2, **kwargs):
    """
    A file stream transform IO utility function.

    :param src: original file path
    :param dst: destination file path
    :param converter: binary content converter function
    :param overwrite: default False,
    :param stream: default True, if True, use stream IO mode, chunksize has to
      be specified.
    :param chunksize: default 1MB
    """
    if not overwrite:  # pragma: no cover
        if Path(dst).exists():
            raise EnvironmentError("'%s' already exists!" % dst)

    with open(src, "rb") as f_input:
        with open(dst, "wb") as f_output:
            if stream:
                # fix chunksize to a reasonable range
                if chunksize > 1024 ** 2 * 10:
                    chunksize = 1024 ** 2 * 10
                elif chunksize < 1024 ** 2:
                    chunksize = 1024 ** 2

                # write file
                while 1:
                    content = f_input.read(chunksize)
                    if content:
                        f_output.write(converter(content, **kwargs))
                    else:
                        break
            else:  # pragma: no cover
                f_output.write(converter(f_input.read(), **kwargs))



#!/usr/bin/env python
# -*- coding: utf-8 -*-


class PasswordError(Exception):
    """symmetric encrypt wrong password error.
    """


class SignatureError(Exception):
    """asymmetric encrypt wrong signature error.
    """



#!/usr/bin/env python
# -*- coding: utf-8 -*-


__version__ = "0.0.5"
__short_description__ = "Super easy-to-use encryption and decryption tool"
__license__ = "MIT"
__author__ = "Sanhe Hu"
__author_email__ = "husanhe@gmail.com"
__maintainer__ = "Sanhe Hu"
__maintainer_email__ = "husanhe@gmail.com"
__github_username__ = "MacHu-GWU"


try:
    from .asymmetric import AsymmetricCipher
    from .symmetric import SymmtricCipher
    from .cipher import BaseCipher
except ImportError:  # pragma: no cover
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function, unicode_literals

import io
import os
import time
import base64

from cryptography.fernet import Fernet

from windtalker.cipher import BaseCipher
from windtalker.exc import PasswordError
from windtalker import fingerprint
from windtalker import files
from windtalker import py23


if py23.is_py2:
    input = raw_input


class SymmtricCipher(Fernet, BaseCipher):
    """A symmtric encryption algorithm utility class helps you easily
    encrypt/decrypt text, files and even a directory.

    :param password: The secret password you use to encrypt all your message.
      If you feel uncomfortable to put that in your code, you can leave it
      empty. The system will ask you manually enter that later.

    **中文文档**

    对称加密器。
    """

    _encrypt_chunk_size = 1024 * 1024  # 1 MB
    _decrypt_chunk_size = 1398200  # 1.398 MB
    """Symmtric algorithm needs to break big files in small chunk, and encrypt
    them one by one, and concatenate them at the end. Each chunk has a fixed 
    size. That's what these two attributes for.
    """

    def __init__(self, password=None):
        if password:
            fernet_key = self.any_text_to_fernet_key(password)
            super(SymmtricCipher, self).__init__(fernet_key)
        else:
            self.input_password()

    def any_text_to_fernet_key(self, text):
        """Convert any text to a fernet key for encryption.
        """
        md5 = fingerprint.fingerprint.of_text(text)
        fernet_key = base64.b64encode(md5.encode("utf-8"))
        return fernet_key

    def input_password(self):
        self.set_password(input(
            "Please enter your secret key (case sensitive): "))

    def set_password(self, password):
        self.__init__(password)

    def set_encrypt_chunk_size(self, size):
        if 1024 * 1024 < size < 100 * 1024 * 1024:
            self._encrypt_chunk_size = size
            self._decrypt_chunk_size = len(self.encrypt(b"x" * size))
        else:
            print("encrypt chunk size has to be between 1MB and 100MB")

    @property
    def metadata(self):
        return {"_encrypt_chunk_size": self._encrypt_chunk_size,
                "_decrypt_chunk_size": self._decrypt_chunk_size, }

    def encrypt(self, binary):
        """Encrypt binary data.
        """
        return super(SymmtricCipher, self).encrypt(binary)

    def decrypt(self, binary):
        """Decrypt binary data.
        """
        try:
            return super(SymmtricCipher, self).decrypt(binary)
        except:
            raise PasswordError("Opps, wrong magic word!")
